from optparse import OptionParser
import logging
import os
import re
import smtplib
import sys

from tha.logcheck.splitter import SEPARATOR
from tha.logcheck.splitter import extractor
from tha.logcheck.splitter import splitter


logger = logging.getLogger('logchecker')


class Logfile(object):
    """Read logfile and maintain state on last log line read"""

    def __init__(self, logfile_name, remember_state):
        self.logfile_name = logfile_name
        self.remember_state = remember_state
        self.statefile_name = self.logfile_name + '.logcheckerstate'
        # ^^^ Hardcoded for now.
        self.lastline = 0
        if self.remember_state:
            if os.path.exists(self.statefile_name):
                lines = open(self.statefile_name).readlines()
                if lines:
                    self.lastline = int(lines[0].strip())
                    logger.debug("Previous last line: %s", self.lastline)

    def save_state(self):
        if self.remember_state:
            f = open(self.statefile_name, 'w')
            f.write('%s\n' % self.current_lastline)
            f.close()

    def logfile_lines(self):
        """Return logfile's lines.

        If a state file is present, return just the lines changed since the last
        time we checked the logfile.
        """
        self.lines = [line.strip('\n') for line in
                      open(self.logfile_name).readlines()]
        self.current_lastline = len(self.lines)
        if self.current_lastline < self.lastline:
            # We've got a fresh file, so start again.
            self.lastline = 0
        extracted = self.lines[self.lastline:]
        logger.debug("Extracted %s lines from logfile %s.",
                     len(extracted), self.logfile_name)
        return extracted

    def entries(self):
        return list(splitter(self.logfile_lines()))


def filter_entries(entries, include_warnings, filters, inverse):
    """Return filtered entries ('filters' is a list of lines)"""
    if include_warnings:
        ignore_levels = ['DEBUG', 'INFO']
    else:
        ignore_levels = ['DEBUG', 'INFO', 'WARNING']
    logger.debug("Filtering out error levels %r", ignore_levels)
    entries = [e for e in entries if e['level'] not in ignore_levels]
    logger.debug("%s entries left after level filtering.", len(entries))

    if not filters:
        logger.debug("No filters, so returning all entries.")
        return entries
    filters = [f for f in filters if f and not f.startswith('#')]
    filters = [re.compile(f) for f in filters]
    result = []
    for entry in entries:
        matched = False
        content = ' '.join(entry['content'])
        # ^^^ multiline content joined to a single string.
        for f in filters:
            if f.search(content):
                matched = True
                continue
        if not matched and not inverse:
            result.append(entry)
        if matched and inverse:
            result.append(entry)
    return result


def format_output(entries):
    """Return formatted output"""
    output = []
    for entry in entries:
        output.append(SEPARATOR)
        output.append('\n'.join(entry['content']))
    return '\n'.join(output)


def return_output(output, email, mailhost, logfile_name, subjectprefix):
    """Email or print the output"""
    if not email:
        print output
        return
    if not subjectprefix:
        subjectprefix = ''
    else:
        subjectprefix += ' '
    host = smtplib.SMTP(host=mailhost)
    to = email
    subject = '%sLogcheck results for %s' % (subjectprefix, logfile_name)
    # Don't want to waste an extra option on the 'from' address for now.
    from_ = 'logchecker@' + email.split('@')[1]
    msg = "From: %s\nTo: %s\nSubject: %s\n\n%s" % (
        from_, to, subject, output)
    logger.debug("Mailing %s about %r", to, subject)
    host.sendmail(from_, to, msg)


def main():
    usage = 'usage: %prog [options] logfile'
    parser = OptionParser(usage=usage)
    parser.add_option(
        '-e', '--email',
        help='E-mail results to ADDRESS (output is printed to stdout otherwise).',
        metavar='ADDRESS')
    parser.add_option(
        '--subjectprefix',
        help='add PREFIX to E-mail subject line.',
        metavar='PREFIX')
    parser.add_option(
        '-f', '--filter',
        help='Read extra filters from FILE. Each line is a regex.',
        metavar='FILE')
    parser.add_option(
        '-i', '--inverse',
        help="Use the filter file in inverse operation (for debugging).",
        action='store_true', default=False)
    parser.add_option(
        '-m', '--mailhost',
        help="""Outgoing e-mail host (defaults to localhost). Use
        'HOSTNAME:PORT' if you need to specify the port number.""",
        default='localhost', # This is smtplib's default, specified here, too.
        metavar='HOSTNAME',)
    parser.add_option(
        '-s', '--no-state',
        help="Do not store last line read, but read the whole logfile.",
        dest='remember_state',
        action='store_false',
        default=True)
    parser.add_option('-v', '--verbose', action='store_true', default=False)
    parser.add_option(
        '-w', '--include-warnings', action='store_true', default=False,
        help="Do not filter out the warning level")
    (options, args) = parser.parse_args()

    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.WARN)
    if not len(args) > 0:
        parser.print_help()
        sys.exit(1)

    logfile_name = args[0]
    logfile = Logfile(logfile_name, options.remember_state)
    entries = logfile.entries()
    logger.debug("Extracted %s entries.", len(entries))
    entries = [extractor(entry) for entry in entries]
    if options.filter:
        filters = [line.strip() for line in open(options.filter).readlines()]
    else:
        filters = None
    entries = filter_entries(entries,
                             options.include_warnings,
                             filters,
                             options.inverse)
    logger.debug("After filtering, we have %s entries.", len(entries))
    if not entries:
        return
    output = format_output(entries)
    return_output(output, options.email, options.mailhost, logfile_name, options.subjectprefix)
    logfile.save_state()
