import sys
import logging
from optparse import OptionParser
from operator import itemgetter

from tha.logcheck.splitter import splitter
from tha.logcheck.splitter import extractor
from tha.logcheck.splitter import SEPARATOR


IGNORELEVELS = ['DEBUG', 'INFO', 'WARNING']

logger = logging.getLogger('logsplitter')


def monthSummary(items):
    """Return a string with a montly count of items.
    """

    monthIds = ['%02.f' % (i+1) for i in range(12)]
    months = {}
    for id in monthIds:
        months[id] = 0
    itemMonths = [item['month'] for item in items]
    for month in itemMonths:
        months[month] += 1
    counts = [str(months[id]) for id in monthIds]
    result = '-'.join(counts)
    return result


def makeSummary(items, key):
    foundValues = {}
    for item in items:
        value = item[key]
        if value not in foundValues:
            foundValues[value] = []
        foundValues[value].append(item)
    summary = []
    for foundValue in foundValues:
        items = foundValues[foundValue]
        summary.append({'name': foundValue.strip(),
                        'count': len(items),
                        'items': items,
                        'months': monthSummary(items)})
    summary.sort(key=itemgetter('count'))
    logger.info("Summary of %ss that have been found:", key)
    for item in summary:
        logger.info("%s times %s. (%s)", item['count'], item['name'],
                 item['months'])
    return summary


def filterFromSummary(summary, key):
    subset = [e['items'] for e in summary
              if e['name'] == key]
    if not subset:
        logger.critical("Nothing found for key %s.",
                     key)
        sys.exit(1)
    subset = subset[0] # un-list the nested list
    return subset


def writeErrors(fileName, errors):
    outfile = open(fileName, 'w')
    for error in errors:
        outfile.write('%s\n' % SEPARATOR)
        outfile.write(''.join(error['content']))


def doSomething(entries, detailsAbout=None):
    structuredErrors = [entry for entry in entries
                  if entry['error']]
    strangeErrors = [entry for entry in entries
                     if not entry['error']]
    writeErrors('strange.txt', strangeErrors)
    logger.info("%s structured error messages.", len(structuredErrors))
    logger.info("%s strange error messages.", len(strangeErrors))
    categorisedErrors = makeSummary(structuredErrors, 'error')
    if detailsAbout:
        logger.info('====================')
        subset = filterFromSummary(categorisedErrors, detailsAbout)
        #summary = makeSummary(subset, 'errortext')
        summary = makeSummary(subset, 'title')
        writeErrors('out.txt', subset)


def main():
    usage = 'usage: %prog [options] logfile'
    parser = OptionParser(usage=usage)
    parser.add_option(
        '-e', '--focus-on-error',
        help="Focus on one specific error, omitting the others.",
        metavar='ERROR')
    parser.add_option(
        '-v', '--verbose', action='store_true', default=False)
    parser.add_option(
        '-w', '--include-warnings', action='store_true', default=False,
        help="Do not filter out the warning level")
    (options, args) = parser.parse_args()
    if options.verbose:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)
    if not len(args) > 0:
        parser.print_help()
        sys.exit(1)

    logfile_name = args[0]
    lines = open(logfile_name, 'r').readlines()
    entries = [extractor(e) for e in splitter(lines)]
    logger.info("Extracted %s entries in total.", len(entries))
    if options.include_warnings:
        ignore_levels = ['DEBUG', 'INFO']
    else:
        ignore_levels = ['DEBUG', 'INFO', 'WARNING']
    entries = [e for e in entries if e['level'] not in ignore_levels]
    logger.info("%s entries left after filtering log levels.",
             len(entries))
    doSomething(entries, detailsAbout=options.focus_on_error)
